==> manifest-merger-30.3.0-sources.jar: com/android/manifmerger/XmlLoader.java <==
/*
 * Copyright (C) 2014 The Android Open Source Project
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.android.manifmerger;

import com.android.annotations.NonNull;
import com.android.annotations.Nullable;
import com.android.ide.common.blame.SourceFile;
import com.android.resources.NamespaceReferenceRewriter;
import com.android.utils.PositionXmlParser;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.xml.sax.SAXException;

import javax.xml.parsers.ParserConfigurationException;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;

import static com.android.manifmerger.PlaceholderHandler.KeyBasedValueResolver;

/**
 * Responsible for loading XML files.
 */
public final class XmlLoader {

    private XmlLoader() {}

    /**
     * Loads an xml file without doing xml validation and return a {@link XmlDocument}
     *
     * @param displayName the xml file display name.
     * @param xmlFile the xml file.
     * @param namespace the namespace, used to create or shorten fully qualified class names. If
     *     null, the manifest's package name is used as the namespace instead.
     * @return the initialized {@link XmlDocument}
     */
    @NonNull
    public static XmlDocument load(
            @NonNull KeyResolver<String> selectors,
            @NonNull KeyBasedValueResolver<ManifestSystemProperty> systemPropertyResolver,
            @NonNull String displayName,
            @NonNull File xmlFile,
            @NonNull InputStream inputStream,
            @NonNull XmlDocument.Type type,
            @Nullable String namespace,
            @NonNull DocumentModel<ManifestModel.NodeTypes> model,
            boolean rewriteNamespaces)
            throws IOException, SAXException, ParserConfigurationException {
        Document domDocument = PositionXmlParser.parse(inputStream);
        Element rootElement = domDocument.getDocumentElement();
        @Nullable
        final String namespaceOrPackageName =
                namespace != null ? namespace : rootElement.getAttribute("package");
        if (rewriteNamespaces) {
            new NamespaceReferenceRewriter(
                            namespaceOrPackageName, (String t, String n) -> namespaceOrPackageName)
                    .rewriteManifestNode(rootElement, true);
        }
        return new XmlDocument(
                new SourceFile(xmlFile, displayName),
                selectors,
                systemPropertyResolver,
                domDocument.getDocumentElement(),
                type,
                namespaceOrPackageName,
                model);
    }

    /**
     * Loads a xml document from its {@link String} representation without doing xml validation and
     * return a {@link XmlDocument}
     *
     * @param sourceFile the source location to use for logging and record collection.
     * @param xml the persisted xml.
     * @param namespace the namespace, used to create or shorten fully qualified class names. If
     *     null, the manifest's package name is used as the namespace instead.
     * @return the initialized {@link XmlDocument}
     * @throws IOException this should never be thrown.
     * @throws SAXException if the xml is incorrect
     * @throws ParserConfigurationException if the xml engine cannot be configured.
     */
    @NonNull
    public static XmlDocument load(
            @NonNull KeyResolver<String> selectors,
            @NonNull KeyBasedValueResolver<ManifestSystemProperty> systemPropertyResolver,
            @NonNull SourceFile sourceFile,
            @NonNull String xml,
            @NonNull XmlDocument.Type type,
            @Nullable String namespace,
            @NonNull DocumentModel<ManifestModel.NodeTypes> model)
            throws IOException, SAXException, ParserConfigurationException {
        Document domDocument = PositionXmlParser.parse(xml);
        Element rootElement = domDocument.getDocumentElement();
        @Nullable
        final String namespaceOrPackageName =
                namespace != null ? namespace : rootElement.getAttribute("package");
        return new XmlDocument(
                sourceFile,
                selectors,
                systemPropertyResolver,
                domDocument.getDocumentElement(),
                type,
                namespaceOrPackageName,
                model);
    }
}

==> manifest-merger-31.7.2-sources.jar: com/android/manifmerger/XmlLoader.java <==
/*
 * Copyright (C) 2014 The Android Open Source Project
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.android.manifmerger;

import static com.android.manifmerger.PlaceholderHandler.KeyBasedValueResolver;

import com.android.annotations.NonNull;
import com.android.annotations.Nullable;
import com.android.ide.common.blame.SourceFile;
import com.android.resources.NamespaceReferenceRewriter;
import com.android.utils.PositionXmlParser;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import javax.xml.parsers.ParserConfigurationException;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.xml.sax.SAXException;

/**
 * Responsible for loading XML files.
 */
public final class XmlLoader {

    private XmlLoader() {}

    /**
     * Loads an xml file without doing xml validation and return a {@link XmlDocument}
     *
     * @param displayName the xml file display name.
     * @param xmlFile the xml file.
     * @param namespace the namespace, used to create or shorten fully qualified class names. If
     *     null, the manifest's package name is used as the namespace instead.
     * @return the initialized {@link XmlDocument}
     */
    @NonNull
    public static XmlDocument load(
            @NonNull KeyResolver<String> selectors,
            @NonNull KeyBasedValueResolver<ManifestSystemProperty> systemPropertyResolver,
            @NonNull String displayName,
            @NonNull File xmlFile,
            @NonNull InputStream inputStream,
            @NonNull XmlDocument.Type type,
            @Nullable String namespace,
            @NonNull DocumentModel<ManifestModel.NodeTypes> model,
            boolean rewriteNamespaces)
            throws IOException, SAXException, ParserConfigurationException {
        Document domDocument = PositionXmlParser.parse(inputStream);
        return load(
                domDocument,
                selectors,
                systemPropertyResolver,
                displayName,
                xmlFile,
                type,
                namespace,
                model,
                rewriteNamespaces);
    }

    /**
     * Build an unvalidated {@link XmlDocument} object from given DOM Document Also see overload
     * {@link XmlLoader#load(KeyResolver, KeyBasedValueResolver, String, File, InputStream,
     * XmlDocument.Type, String, DocumentModel, boolean)} for more details.
     *
     * @param domDocument Manifest Document object.
     */
    @NonNull
    public static XmlDocument load(
            @NonNull Document domDocument,
            @NonNull KeyResolver<String> selectors,
            @NonNull KeyBasedValueResolver<ManifestSystemProperty> systemPropertyResolver,
            @NonNull String displayName,
            @NonNull File xmlFile,
            @NonNull XmlDocument.Type type,
            @Nullable String namespace,
            @NonNull DocumentModel<ManifestModel.NodeTypes> model,
            boolean rewriteNamespaces)
            throws IOException, SAXException, ParserConfigurationException {
        Element rootElement = domDocument.getDocumentElement();
        @Nullable
        final String namespaceOrPackageName =
                namespace != null ? namespace : rootElement.getAttribute("package");
        if (rewriteNamespaces) {
            new NamespaceReferenceRewriter(
                            namespaceOrPackageName, (String t, String n) -> namespaceOrPackageName)
                    .rewriteManifestNode(rootElement, true);
        }
        return new XmlDocument(
                new SourceFile(xmlFile, displayName),
                selectors,
                systemPropertyResolver,
                domDocument.getDocumentElement(),
                type,
                namespaceOrPackageName,
                model);
    }

    /**
     * Loads a xml document from its {@link String} representation without doing xml validation and
     * return a {@link XmlDocument}
     *
     * @param sourceFile the source location to use for logging and record collection.
     * @param xml the persisted xml.
     * @param namespace the namespace, used to create or shorten fully qualified class names. If
     *     null, the manifest's package name is used as the namespace instead.
     * @return the initialized {@link XmlDocument}
     * @throws IOException this should never be thrown.
     * @throws SAXException if the xml is incorrect
     * @throws ParserConfigurationException if the xml engine cannot be configured.
     */
    @NonNull
    public static XmlDocument load(
            @NonNull KeyResolver<String> selectors,
            @NonNull KeyBasedValueResolver<ManifestSystemProperty> systemPropertyResolver,
            @NonNull SourceFile sourceFile,
            @NonNull String xml,
            @NonNull XmlDocument.Type type,
            @Nullable String namespace,
            @NonNull DocumentModel<ManifestModel.NodeTypes> model)
            throws IOException, SAXException, ParserConfigurationException {
        Document domDocument = PositionXmlParser.parse(xml);
        Element rootElement = domDocument.getDocumentElement();
        @Nullable
        final String namespaceOrPackageName =
                namespace != null ? namespace : rootElement.getAttribute("package");
        return new XmlDocument(
                sourceFile,
                selectors,
                systemPropertyResolver,
                domDocument.getDocumentElement(),
                type,
                namespaceOrPackageName,
                model);
    }
}
